
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

#include "rendererFactory.h"

rendererFactory::rendererFactory(HINSTANCE hInst)
{
	m_hInst = hInst;
	m_pRenderer = NULL;

}

rendererFactory::~rendererFactory()
{
	destroyInterface();
}

HRESULT rendererFactory::initInterface(TCHAR *szIniFile, HWND hWnd)
{
	
	//just to be even more clever, we'll pass in an .ini
	//file to this Factory object, which will give us the 
	//added bonus of changing the renderer type on the fly,
	//rather than having to recompile each time..

	//we COULD create a class to encapsulate the Get/WritePrivateProfileString
	//behaviour, but for some clarity, I'll just stick to the
	//bare win32 calls...
	TCHAR currentPath[MAX_PATH];
	ZeroMemory(&currentPath, sizeof(MAX_PATH));
	GetCurrentDirectory(MAX_PATH, currentPath);
	strcat(currentPath, "\\");
	strcat(currentPath, szIniFile);

	DWORD dwRetVal = 0;
	TCHAR szType[MAX_PATH];
	DWORD dwSizeVal = MAX_PATH;
	dwRetVal = GetPrivateProfileString("Video",    // section name
									  "Renderer",       // key name
									  "OpenGL", // default string
									  szType,      // destination buffer
									  dwSizeVal,       // size of destination buffer
									  currentPath);   // initialization file name


	if(strcmp(szType, "OpenGL") == 0)
	{
		//we're using OpenGL for our renderer
		m_hDLL = LoadLibraryEx("GameOGLRenderer.dll",NULL,0);
		if(!m_hDLL)
		{
			MessageBox(NULL,
					   "Error loading up GameOGLRenderer.dll",
					   "Fatal Error",
					   MB_OK | MB_ICONERROR);

			return E_FAIL;
		}

		//MessageBox(NULL, "Loaded up OpenGL renderer", "Info", MB_OK | MB_ICONINFORMATION);
		SetWindowText(hWnd, "Using OpenGL Renderer Interface");

	}
	else
	{
		//we're using Direct3D for our renderer
		m_hDLL = LoadLibraryEx("GameD3DRenderer.dll",NULL,0);
		if(!m_hDLL)
		{
			MessageBox(NULL,
					   "Error loading up GameD3DRenderer.dll",
					   "Fatal Error",
					   MB_OK | MB_ICONERROR);

			return E_FAIL;
		}

		//MessageBox(NULL, "Loaded up Direct3D renderer", "Info", MB_OK | MB_ICONINFORMATION);
		SetWindowText(hWnd, "Using Direct3D Renderer Interface");


	}

	//okay now that we've got our dll loaded, let's get a pointer
	//to our createInterface function
	CREATERENDERERINTERFACE CreateInterface = 0;
	HRESULT hr;

	CreateInterface = (CREATERENDERERINTERFACE)GetProcAddress(m_hDLL,
															  "createRendererInterface");


	hr = CreateInterface(&m_pRenderer);
	if(FAILED(hr)){
		MessageBox(NULL,
				   "Error using our createInterface function",
				   "FATAL Error", MB_OK | MB_ICONERROR);
		m_pRenderer = NULL;
		return E_FAIL;
	}

	return S_OK;
}

void rendererFactory::destroyInterface()
{
	DESTROYRENDERERINTERFACE DestroyInterface = 0;
	HRESULT hr;

	//function pointer to our destroyRendererInterface function
	//within our DLL
	DestroyInterface = (DESTROYRENDERERINTERFACE)GetProcAddress(m_hDLL,
																"destroyRendererInterface");
	//call the DLL's method to properly destroy our rendererInterface
	//object
	hr = DestroyInterface(&m_pRenderer);
	if(FAILED(hr)){
		MessageBox(NULL,
				   "Error using our destroyInterface function",
				   "GameFramework Error", MB_OK);

		m_pRenderer = NULL;
	}
}